/*
   The MIT License (MIT)
   -----------------------------------------------------------------------------

   Copyright (c) 2015-2019 javafx.ninja <info@javafx.ninja>

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.

*/

package ninja.javafx.smartcsv.validation;

import java.util.ArrayList;
import java.util.List;

/**
 * This class holds all the error messages
 * for a single cell and the information in
 * which row the cell is
 */
public class ValidationError {

    private List<ValidationMessage> messages = new ArrayList<>();
    private Integer lineNumber;
    private String column = "";

    private ValidationError(Integer lineNumber) {
        this.lineNumber = lineNumber;
    }

    public static ValidationError withLineNumber(int lineNumber) {
        return new ValidationError(lineNumber);
    }

    public static ValidationError withoutLineNumber() {
        return new ValidationError(-1);
    }

    public ValidationError column(String column) {
        this.column = column;
        return this;
    }

    public Integer getLineNumber() {
        return lineNumber;
    }

    public String getColumn() {
        return column;
    }

    public List<ValidationMessage> getMessages() {
        return messages;
    }

    public ValidationError add(String key, String... parameters) {
        messages.add(new ValidationMessage(key, parameters));
        return this;
    }

    public boolean isEmpty() {
        return messages.isEmpty();
    }
}
